<footer>
    Powered by <a href="https://bovine.codeberg.page/cattle_grid/">cattle_grid</a>
    and its <a href="https://bovine.codeberg.page/cattle_grid/extensions/html_display/">html display</a>
    extension.<br>
    Version {{version}}
</footer>