<!DOCTYPE html>
<html>

<head>
    {% include 'style.html.j2' %}

    <style>
        div.date {
            text-align: right;
            font-size: small;
            font-weight: bold;
        }
    </style>
    <title>@{{ name }}</title>
</head>

<body>
    <header>
        <h1><a href="/@{{ name }}">@{{ name }}</a></h1>
    </header>
    <article>{{ content }}</article>

    {% if replies %}
    <article>
        <h3>Replies</h3>
        <ul>
            {% for x in replies %}
            <li>{{ x }}</li>
            {% endfor %}
        </ul>
    </article>
    {% endif %}
    {% if shares %}
    <article>
        <h3>Shares</h3>
        <ul>
            {% for x in shares %}
            <li>{{ x }}</li>
            {% endfor %}
        </ul>
    </article>
    {% endif %}
    {% if likes %}
    <article>
        <h3>Likes</h3>
        <ul>
            {% for x in likes %}
            <li>{{ x }}</li>
            {% endfor %}
        </ul>
    </article>
    {% endif %}
    {% include 'footer.html.j2' %}
</body>

</html>