<style>
    body {
        background-color: aquamarine;
    }

    header,
    article,
    footer {
        margin-left: calc(50% - 350px);
        width: 700px;
        background-color: white;
        color: black;
        border: 2px black solid;
        padding: 10px;
    }

    footer {
        background-color: lightgrey;
        margin-top: 40px;
    }

    article {
        margin-top: 20px;
    }
</style>